/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package northwind.controladores;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import northwind.modelos.ConexionBD;
import northwind.modelos.GestorOrders;
import northwind.modelos.ModeloTablaDetalle;
import northwind.modelos.ModeloTablaOrders;
import northwind.modelos.Order;
import northwind.vistas.VentanaLogin;
import northwind.vistas.VentanaOrders;

public class ControladorOrders implements IControladorOrders {
    private VentanaOrders ventanaOrders;
    private ConexionBD conexion;
    
    private static ControladorOrders instancia;
    private GestorOrders go;
    
    /**
     * Constructor
     * @param ventanaLogin ventana padre
     * @param conexion conexión a la BD
    */    
    private ControladorOrders(VentanaLogin ventanaLogin, ConexionBD conexion) {
        this.ventanaOrders = new VentanaOrders(ventanaLogin, this);
        //muestra VentanaOrders
        
        this.configurarTablaOrders();
        //configura la tabla de órdenes
        
        this.conexion = conexion;
        this.go = GestorOrders.crear(this.conexion);
                
        this.mostrarOrdenes();
        //muestra todas las órdenes  
        
        this.ventanaOrders.setVisible(true);
    }
    
    /**
     * Método que permite crear una única instancia de ControladorOrders
    */    
    public static ControladorOrders crear(VentanaLogin ventanaLogin, ConexionBD conexion) {
        if (instancia == null) {
            instancia = new ControladorOrders(ventanaLogin, conexion);
        }
        return instancia;
    }
    
    /**
     * Método que permite volver a crear una instancia de ControladorOrders
    */        
    public static void destruir() {
        instancia = null;
    }
    
    /**
     * Devuelve la ventana de órdenes
     * @return VentanaOrders  - ventana de órdenes
    */        
    public VentanaOrders getVentanaOrders() {
        return this.ventanaOrders;
    }
    
    /**
     * Configura la tabla de órdenes agregándole un listener para detectar cuando se hace una selección
    */            
    public void configurarTablaOrders() {
        JTable tablaOrders = this.ventanaOrders.getTablaOrders();
        tablaOrders.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    //En esta tabla se puede seleccionar sólo un elemento a la vez, 
                    //por lo que cuando se realiza una nueva selección, 
                    //el elemento seleccionado anteriormente se convierte en no seleccionado, 
                    //disparándose 2 eventos cada vez que se selecciona un elemento diferente
                    //Para evitar responder al evento cuando un elemento deja de estar seleccionado
                    //y luego cuando otro queda seleccionado, se comprueba que esta secuencia de eventos
                    //esté terminada mediante getValueIsAdjusting()
                    int fila = tablaOrders.getSelectedRow();
                    ModeloTablaOrders mto = (ModeloTablaOrders)tablaOrders.getModel();
                    Order unaOrder = mto.getOrder(fila);
                    unaOrder.leerOrderDetails();
                    
                    JTable tablaOrderDetails = ventanaOrders.getTablaDetalle();
                    ModeloTablaDetalle mtd = new ModeloTablaDetalle(unaOrder);
                    tablaOrderDetails.setModel(mtd);
                    
                    JTextField campoTotal = ventanaOrders.getTxtTotal();
                    float total = unaOrder.calcularTotal();
                    campoTotal.setText(Float.toString(total));
                }
            }
        });        
    }    

    /**
     * Muestra todas las órdenes
    */                                                                
    private void mostrarOrdenes() {
        ModeloTablaOrders mto = new ModeloTablaOrders(this.go);
        JTable tablaOrders = this.getVentanaOrders().getTablaOrders();
        tablaOrders.setModel(mto);
        
        if (tablaOrders.getRowCount() > 0) {
            tablaOrders.setRowSelectionInterval(0, 0);
            //selecciona la primer order de la tabla
        }        
    }

    @Override
    public void botonVolverClick() {
        this.ventanaOrders.dispose();
    }
}
