/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package northwind.modelos;

import java.sql.DriverManager;
import javax.swing.JOptionPane;

/**
 * Clase para realizar una conexión a un servidor MySQL
*/       
public class ConexionMySQL extends ConexionBD { 
    private static ConexionMySQL instancia;
    
    /**
     * Constructor
     * Este constructor asigna el valor a las variables url, driver, servidor, puerto, bd, usuario y clave (definidas en TipoConexion)
     * y luego carga la implementación del driver en memoria (a este proceso se lo llama "Registro del driver)
     * Después de cargar el driver en memoria, se establece una conexión a la BD mediante el método DriverManager.getConnection()
     * http://www.tutorialspoint.com/jdbc/jdbc-db-connections.htm
     * @param servidor servidor al cual conectarse
     * @param puerto puerto en el cual escucha el servidor
     * @param bd base de datos a la cual conectarse
     * @param usuario usuario con el cual conectarse
     * @param clave clave del usuario que hace la conexión
    */    
    private ConexionMySQL(String servidor, String puerto, String bd, String usuario, String clave) {
        super("jdbc:mysql://", "com.mysql.jdbc.Driver");
        try {
            Class.forName(this.getDriver());
//            this.setConnection(DriverManager.getConnection(this.getURL() + servidor + ":" + puerto + "/" + bd, usuario, clave));
            this.setConnection(DriverManager.getConnection(this.getURL() + servidor + ":" + puerto + "/" + bd + "?noAccessToProcedureBodies=true", usuario, clave));
            //Si no se especifica la opción noAccessToProcedureBodies=true, aparece el siguiente mensaje de error:
            //"User does not have access to metadata required to determine stored procedure parameter types. 
            //If rights can not be granted, configure connection with "noAccessToProcedureBodies=true" to have driver generate parameters that represent INOUT strings irregardless of actual parameter types."
            //Otra forma de solucionar el error es otorgándole el permiso SELECT al usuario sobre la tabla mysql.proc (información sobre los procedimientos almacenados):
            //GRANT SELECT ON mysql.proc TO 'user'@'localhost';
            //La desventaja de esta alternativa es que se le está dando al usuario permisos para leer la tabla antera,
            //la cual contiene información sobre todos los procedimientos en todas las BD (incluso el código fuente)
            //http://stackoverflow.com/questions/986628/cant-execute-a-mysql-stored-procedure-from-java
        }
        catch (Exception cnfe) {
            String mensajeError = "Error de conexión\n" + "Servidor: " + servidor + "\nPuerto: " + puerto + "\nBD: " + bd + "\nUsuario: " + usuario;
            JOptionPane.showMessageDialog(null, mensajeError, "Agenda", JOptionPane.ERROR_MESSAGE);        
            System.exit(0);
        }
    }  
    
    /**
     * Método estático que permite crear una única instancia de ConexionMySQL
     * @param servidor servidor al cual conectarse
     * @param puerto puerto en el cual escucha el servidor (3306 por defecto)
     * @param bd base de datos a la cual conectarse
     * @param usuario usuario con el cual conectarse
     * @param clave clave del usuario que hace la conexión
     * @return ConexionMySQL
    */                                                                    
    public static ConexionMySQL crear(String servidor, String puerto, String bd, String usuario, String clave) {
        if (instancia == null) 
            instancia = new ConexionMySQL(servidor, puerto, bd, usuario, clave);
        return instancia;        
    }       
}
