/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package northwind.modelos;

import java.sql.DriverManager;
import javax.swing.JOptionPane;

/**
 * Clase para realizar una conexión a un servidor SQL Server
*/       
public class ConexionSQLServer extends ConexionBD {
    private static ConexionSQLServer instancia;
    
    /**
     * Constructor
     * Este constructor asigna el valor a las variables url, driver, servidor, puerto, bd, usuario y clave (definidas en TipoConexion)
     * y luego carga la implementación del driver en memoria (a este proceso se lo llama "Registro del driver)
     * Después de cargar el driver en memoria, se establece una conexión a la BD mediante el método DriverManager.getConnection()
     * http://www.tutorialspoint.com/jdbc/jdbc-db-connections.htm
     * @param servidor servidor al cual conectarse
     * @param puerto puerto en el cual escucha el servidor (1433 por defecto)
     * @param bd base de datos a la cual conectarse
     * @param usuario usuario con el cual conectarse
     * @param clave clave del usuario que hace la conexión
    */        
    private ConexionSQLServer(String servidor, String puerto, String bd, String usuario, String clave) {
        super("jdbc:sqlserver://", "com.microsoft.sqlserver.jdbc.SQLServerDriver");
        try {         
            Class.forName(this.getDriver());
            this.setConnection(DriverManager.getConnection(this.getURL() + servidor + ":" + puerto + ";DatabaseName=" + bd, usuario, clave));            
        }
        catch (Exception e) {
            String mensajeError = "Error de conexión\n" + "Servidor: " + servidor + "\nPuerto: " + puerto + "\nBD: " + bd + "\nUsuario: " + usuario;
            JOptionPane.showMessageDialog(null, mensajeError, "Northwind", JOptionPane.ERROR_MESSAGE);                    
            System.exit(0);
        }
    }
    
    /**
     * Método estático que permite crear una única instancia de ConexionSQLServer
     * @param servidor servidor al cual conectarse
     * @param puerto puerto en el cual escucha el servidor
     * @param bd base de datos a la cual conectarse
     * @param usuario usuario con el cual conectarse
     * @param clave clave del usuario que hace la conexión
     * @return ConexionSQLServer
    */                                                                
    public static ConexionSQLServer crear(String servidor, String puerto, String bd, String usuario, String clave) {
        if (instancia == null) 
            instancia = new ConexionSQLServer(servidor, puerto, bd, usuario, clave);
        return instancia;        
    }           
       
}
