/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package northwind.modelos;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.List;

public class GestorOrders {   
    private List<Order> orders = new ArrayList<>();
    private static GestorOrders instancia; 
    private ConexionBD conexion;
        
    /**
     * Constructor
     * @param conexion conexión a la BD
    */
    private GestorOrders(ConexionBD conexion) {
        this.conexion = conexion;
        if (this.orders.isEmpty())
            this.leerOrders();
    }
    
    /**
     * Método estático que permite crear una única instancia de GestorOrdersBD
     * @param conexion conexión a la BD
     * @return GestorOrdersBD
    */                                                            
    public static GestorOrders crear(ConexionBD conexion) {
        if (instancia == null) 
            instancia = new GestorOrders(conexion);
        return instancia;        
    }   
    
    /**
     * Lee las orders de la BD
     * Si hay algún problema con la BD, devuelve null
     * @return List<Persona>  - lista con las personas cuyo apellido empiece con la cadena especificada
    */                                                        
    private void leerOrders() {
        CallableStatement cstmt = null;
        try {
            Connection cnn = this.conexion.getConnection();
            String sql = "{call Ordenes()}";
            cstmt = cnn.prepareCall(sql);
            ResultSet rs = cstmt.executeQuery();
            GestorCustomers gc = GestorCustomers.crear();
            GestorEmployees ge = GestorEmployees.crear();
            while (rs.next()) {
                int orderID = rs.getInt("OrderID");
                
                String customerID = rs.getString("CustomerID");
                String compania = rs.getString(3);
                
                int employeeID = rs.getInt("EmployeeID");
                String apellido = rs.getString("Apellido");
                String nombre = rs.getString("Nombre");
                LocalDate fecha = Instant.ofEpochMilli(rs.getDate("Fecha").getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
                
                Customer cust = gc.crearCustomer(customerID, compania);
                Employee emp = ge.crearEmployee(employeeID, apellido, nombre);
                Order unaOrder = new Order(orderID, cust, emp, fecha, this.conexion);
                this.orders.add(unaOrder);                   
            }
            rs.close();
        }
        catch(Exception e) {
            e.printStackTrace();
        } 
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                            
    }
        
    /**
     * Devuelve la order correspondiente a la posición especificacicada
     * Si no hay orders, o la posición especificada es incorrecta, devuelve null
     * @param index posición para buscar una order
     * @return Order  - order correspondiente a la posición especificacicada
    */                                                                        
    public Order getOrder(int index) {
        if ((index < 0) || (index >= this.orders.size()))
            return null;
        if (!this.orders.isEmpty())
            return this.orders.get(index);
        else 
            return null;
    }
    
    /**
     * Devuelve la cantidad de orders
     * @return int  - cantidad de orders
    */                                                                    
    public int cantOrders() {
        return this.orders.size();
    }

    /**
     * Devuelve las orders
     * @return List<Order>  - lista de orders
    */                                                                
    public List<Order> getOrders() {
        return this.orders;
    }
    
    public void mostrar() {
        for(Order o : this.orders)
            o.mostrar();
    }
}
