/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package northwind.modelos;

import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;

/**
 * Clase para manejar la tabla de detalles
 */                    
public class ModeloTablaDetalle extends AbstractTableModel {
    //colección para guardar los nombres de las columnas
    private List<String> nombresColumnas = new ArrayList<>();        

    //los datos los saca de una orden
    private Order order;
    
    /**
     * Constructor
     * @param order order a la cual se le quiere ver el detalle
    */                                                            
    public ModeloTablaDetalle(Order order) {
        this.order = order;
        this.nombresColumnas.add("Producto");
        this.nombresColumnas.add("Precio");
        this.nombresColumnas.add("Cantidad");
        this.nombresColumnas.add("Descuento");
        this.nombresColumnas.add("Total");
    }   

    /**
     * Obtiene el valor de la celda especificada
     * @param fila fila de la celda
     * @param columna columna de la celda
     * @return Object  - valor de la celda
    */                           
    @Override
    public Object getValueAt(int fila, int columna) {
        OrderDetail orderDetail = this.order.getOrderDetail(fila);
        if (orderDetail != null) {
            switch(columna) {
                case 0: return orderDetail.getProducto().getProductName();
                case 1: return orderDetail.getUnitPrice();
                case 2: return orderDetail.getQuantity();
                case 3: return orderDetail.getDiscount();
                case 4: return orderDetail.getTotal();
                default: return orderDetail.getProducto().getProductName();
            }
        }
        else
            return null;
    }

    /**
     * Obtiene la cantidad de columnas de la tabla
     * @return int  - cantidad de columnas de la tabla
    */                                
    @Override
    public int getColumnCount() {
        return this.nombresColumnas.size();
    }

    /**
     * Obtiene la cantidad de filas de la tabla
     * @return int  - cantidad de filas de la tabla
    */                            
    @Override
    public int getRowCount() {
        return this.order.cantOrderDetails();
    }

    /**
     * Obtiene el nombre de una columna
     * @param columna columna sobre la que se quiere obtener el nombre
     * @return String  - nombre de la columna especificada
    */                            
    @Override
    public String getColumnName(int columna) {
         return this.nombresColumnas.get(columna);
    }        

    /**
     * Redibuja totalmente la tabla y avisa a todos los detectores que las celdas de la misma 
     * pueden haber cambiado y que se podrían haber agregado o quitado filas de la tabla
    */        
    public void refrescar() {
        fireTableChanged(null);
    }        
}
