/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package northwind.modelos;

import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;

/**
 * Clase para manejar la tabla de órdenes
 */                    
public class ModeloTablaOrders extends AbstractTableModel {
    //colección para guardar los nombres de las columnas
    private List<String> nombresColumnas = new ArrayList<>();        

    //los datos los saca de GestorOrdenes
    private GestorOrders go;
    
    /**
     * Constructor
    */                                                            
    public ModeloTablaOrders(GestorOrders go) {
        this.go = go;
        this.nombresColumnas.add("# Orden");
        this.nombresColumnas.add("Compañía");
        this.nombresColumnas.add("Empleado");
        this.nombresColumnas.add("Fecha");
    }   

    /**
     * Obtiene el valor de la celda especificada
     * @param fila fila de la celda
     * @param columna columna de la celda
     * @return Object  - valor de la celda
    */                           
    @Override
    public Object getValueAt(int fila, int columna) {
        Order unaOrder = this.go.getOrder(fila);
        if (unaOrder != null) {
            switch(columna) {
                case 0: return unaOrder.getOrderID();
                case 1: return unaOrder.getCustomer().getCompanyName();
                case 2: return unaOrder.getEmployee().getLastName() + ", " + unaOrder.getEmployee().getFirstName();
                case 3: return unaOrder.getOrderDate();
                default: return unaOrder.getOrderID();
            }
        }
        else
            return null;
    }

    /**
     * Obtiene la cantidad de columnas de la tabla
     * @return int  - cantidad de columnas de la tabla
    */                                
    @Override
    public int getColumnCount() {
        return this.nombresColumnas.size();
    }

    /**
     * Obtiene la cantidad de filas de la tabla
     * @return int  - cantidad de filas de la tabla
    */                            
    @Override
    public int getRowCount() {
        return this.go.cantOrders();
    }

    /**
     * Obtiene el nombre de una columna
     * @param columna columna sobre la que se quiere obtener el nombre
     * @return String  - nombre de la columna especificada
    */                            
    @Override
    public String getColumnName(int columna) {
         return this.nombresColumnas.get(columna);
    }        

    /**
     * Redibuja totalmente la tabla y avisa a todos los detectores que las celdas de la misma 
     * pueden haber cambiado y que se podrían haber agregado o quitado filas de la tabla
    */        
    public void refrescar() {
        fireTableChanged(null);
    }    
    
    /**
     * Devuelve la order correspondiente a la posición especificacicada
     * Si no hay orders, o la posición especificada es incorrecta, devuelve null
     * @param index posición para buscar una order
     * @return Order  - order correspondiente a la posición especificacicada
    */                                                                        
    public Order getOrder(int index) {
        return this.go.getOrder(index);
    }    
}
