/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package northwind.modelos;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

public class Order {
    private int orderID;
    private Customer customer;
    private Employee employee;
    private LocalDate orderDate;
    private List<OrderDetail> orderDetail = new ArrayList<>();
    
    private ConexionBD conexion;
    //Como Order actúa como gestor para OrderDetail, necesita la conexión

    /**
     * Constructor
     * @param orderID ID de la orden
     * @param customer cliente
     * @param employee empleado
     * @param orderDate fecha de la orden
    */                                                                
    public Order(int orderID, Customer customer, Employee employee, LocalDate orderDate, ConexionBD conexion) {
        this.orderID = orderID;
        this.customer = customer;
        this.employee = employee;
        this.orderDate = orderDate;
        this.conexion = conexion;
    }

    /**
     * Devuelve el ID de la orden
     * @return int  - ID de la orden
    */                                                                    
    public int getOrderID() {
        return this.orderID;
    }

    /**
     * Devuelve el cliente
     * @return Customer  - cliente
    */                                                                        
    public Customer getCustomer() {
        return this.customer;
    }

    /**
     * Devuelve el empleado
     * @return Employee  - empleado
    */                                                                            
    public Employee getEmployee() {
        return this.employee;
    }

    /**
     * Devuelve la fecha de la orden
     * @return LocalDate  - fecha de la orden
    */                                                                            
    public LocalDate getOrderDate() {
        return this.orderDate;
    }      

    /**
     * Devuelve las order details de la orden
     * @return List<OrderDetail>  - order details de la orden
    */                                                                                
    public List<OrderDetail> getOrderDetails() {
        return this.orderDetail;
    }
            
    /**
     * Devuelve la order detail correspondiente a la posición especificacicada
     * Si no hay orders detail, o la posición especificada es incorrecta, devuelve null
     * @param index posición para buscar una order detail
     * @return OrderDetail  - order detail correspondiente a la posición especificacicada
    */                                                                        
    public OrderDetail getOrderDetail(int index) {
        if ((index < 0) || (index >= this.orderDetail.size()))
            return null;
        if (!this.orderDetail.isEmpty())
            return this.orderDetail.get(index);
        else 
            return null;
    }    
    
    /**
     * Devuelve la cantidad de order details
     * @return int  - cantidad de order details
    */                                                                    
    public int cantOrderDetails() {
        return this.orderDetail.size();
    }






    
//    public OrderBD(int orderID, Customer customer, Employee employee, LocalDate orderDate, ConexionBD conexion) {
//        super(orderID, customer, employee, orderDate);
//        this.conexion = conexion;
//    }
    
    /**
     * Lee las order details de esta order
    */                                                                        
    public void leerOrderDetails() {
        CallableStatement cstmt = null;
        try {
            Connection cnn = this.conexion.getConnection();
            String sql = "{call DetalleOrden(?)}";
            cstmt = cnn.prepareCall(sql);
            cstmt.setInt(1, this.getOrderID());
            ResultSet rs = cstmt.executeQuery();
            GestorProducts gp = GestorProducts.crear();
            while (rs.next()) {
                int orderID = rs.getInt("OrderID");
                
                int productID = rs.getInt("ProductID");
                String productName = rs.getString("Producto");
                
                float unitPrice = rs.getFloat("Precio Unitario");
                int quantity = rs.getInt("Cantidad");
                float descuento = rs.getFloat("Descuento");
                float total = rs.getFloat("Total");
                
                Product product = gp.crearProduct(productID, productName);
                this.orderDetail.add(new OrderDetail(product, unitPrice, quantity, descuento, total));
            }
            rs.close();
        }
        catch(Exception e) {
            e.printStackTrace();
        } 
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                                    
    }

    /**
     * Calcula el total de la order
     * @return float  - total de la order
    */                                                                                
    public Float calcularTotal() {
        CallableStatement cstmt = null;
        try {
            Connection cnn = this.conexion.getConnection();
            String sql = "{call TotalOrden(?)}";
            cstmt = cnn.prepareCall(sql);
            cstmt.setInt(1, this.getOrderID());
            ResultSet rs = cstmt.executeQuery();
            rs.next();
            float total = rs.getFloat("Total");              
            rs.close();
            return total;
        }
        catch(Exception e) {
            e.printStackTrace();
            return null;
        } 
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                                    
    }    
    
    public void mostrar() {
        System.out.println("OrderID: " + this.orderID + "\n");
    }
}
